#!/opt/VRTSsfmh/bin/perl
#
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $
#

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i) }

use strict;

require VRTS::Ini;
require VRTS::Paths;
require VRTS::Util;
require VRTS::EXITCODE;
require VRTS::OS;

use constant EXIT_SUCCESS => 0;
use constant EXIT_FAILURE => 1;

my $vbssvc_cmd;
my $logdir;
my $vbs_dir = undef;
my $logfile = 'command.log';
my ($_DO, $_DOE);
my $taskid;
my $subtaskid;
my $source_clus;
my $cmd;

sub discover_vbs_installation
{
	my $vbs_bin_dir = undef;

	if($^O =~ /Win32/i)
	{
		$vbs_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VRTSvbs\\Base" , "HomeDirectory");
		if (not defined $vbs_dir)
		{
			exit(&VRTS::EXITCODE::DIR_NOTFOUND);
		} 
		$vbs_bin_dir = $vbs_dir."\\bin\\";
		$logdir = $vbs_dir."\\log\\";
		$vbssvc_cmd  = $vbs_bin_dir."vbssvc.exe";
	} 
	else 
	{
		$vbssvc_cmd  = "/opt/VRTSvbs/bin/vbssvc";
		$logdir = "/var/VRTSvbs/log/";
	}
	
	if ( ! -f "$vbssvc_cmd") 
	{
		exit(&VRTS::EXITCODE::FILE_NOTFOUND);
	}
	
	if($^O =~ /Win32/i)
	{
		$vbssvc_cmd = "\"" . $vbssvc_cmd . "\""; 
	}
}

sub init_logging
{
	my $log = new VRTS::Logger(
							Module => 'vbslog',
							Logdir => $logdir,
							Level => 'debug',
							Logfile => $logfile,
							UmiString => 'V-0000-0000');
}

sub fail {
	$_DO = $_[0];
	$_DOE = EXIT_FAILURE;
	VRTS::Logger::log('error',"$_DO");
	return $_DO;
}

sub _do {
	my($do,$line,$ret, $pid);
	($do) = (@_);
	
	#Read the file line by line, place each line into variable $line
	# INPUT is the file handle
	if ($^O =~ /Win32/i) {
		# Use IPC::Open3 instead of Open, to capture stderr output on Windows.
		use IPC::Open3;
		$pid = open3(\*STDIN, \*INPUT, \*INPUT, $do) || return fail("Failed to execute command $do");
	} else {
		open(INPUT,"$do 2>&1 |") || return fail("Failed to execute command $do");
	}
	
	while ($line = <INPUT>) { chomp($line); $ret = ($ret) ? "$ret. $line" : $line; }
	close INPUT;
	if ($^O =~ /Win32/i) {
		waitpid $pid, 0;
	}
	
	$_DOE = $? >> 8;
	$_DO = $ret;
	
	VRTS::Logger::log('info',"Executed $do sucessfully. exit code: $_DOE. Output: $_DO");
	return $ret;
}

sub escape_json_string
{
	my $json = shift;
	
	if($^O =~ /Win32/i)
	{
		# Escaping quotes in the JSON message on Win32
		$json =~  s/"/\\"/g;
		# Adding quotes at the beginning and end
		$json = "\"$json\"";
	}
	else 
	{
		# Single quotes in case of Unix
		$json = "\'$json\'";
	}
	
	return $json;
}

sub main {
	discover_vbs_installation();
	init_logging();
	
	my $num_args = scalar(@ARGV);
	if ($num_args != 4) {
		fail("Invalid arguments");
	}
	
	$taskid = $ARGV[0];
	$subtaskid = $ARGV[1];
	$source_clus = $ARGV[2];
	$cmd = $ARGV[3];
	
	_do($cmd);
	
	my $operation_type = "-update-subtasks";
	my $object_type = "Task";
	
	my $json = "{";
	$json .= "\"OperationType\" : \"$operation_type\", ";
	$json .= "\"ObjectType\" : \"$object_type\", ";
	$json .= "\"CLIOperation\" : true, ";
	$json .= "\"Objects\" : [ ";
	$json .= " { ";
	$json .= "\"ObjectName\" : \"$taskid\", ";
	$json .= "\"Attributes\" : [ ";
	$json .= " { \"AttributeName\" : \"SubtaskId\", \"Value\" : $subtaskid }, ";
	$json .= " { \"AttributeName\" : \"SourceCluster\", \"Value\" : \"$source_clus\" }, ";
	$json .= " { \"AttributeName\" : \"Command\", \"Value\" : \"$cmd\" }, ";
	$json .= " { \"AttributeName\" : \"ExitCode\", \"Value\" : $_DOE }, ";
	$json .= " { \"AttributeName\" : \"OutStr\", \"Value\" : \"$_DO\" } ";
	$json .= " ] ";
	$json .= " } ";
	$json .= " ] ";
	$json .= "}";
	
	$json = escape_json_string($json);
	
	my $cmd = "$vbssvc_cmd -send $json";
	_do($cmd);
	
	exit($_DOE);
}

main();
