#!/opt/VRTSsfmh/bin/perl
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
# $Id: execTierCmd.pl,v 1.5 2013/05/30 09:28:44 amukher1 Exp $
#

my $obj_data;
my $tier_module;

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i); }

use strict;
use warnings;
# use FindBin qw($Bin);
use Cwd 'abs_path';
use File::Basename 'dirname';
use File::Spec;
use VRTS::Logger;


sub init {
	my $bin = dirname(abs_path($0));

	push (@INC, File::Spec->catpath(undef, dirname($bin), "lib"));
	require VBSTier::Common;
	{
		no strict 'refs';
		my $mod = "VBSTier::Common";
		$mod->import();
	}

	my @vbs_dirs = discover_vbs_installation();
	init_log('execTierCmd', $vbs_dirs[2]);

	$obj_data = parse_cmdline(
					["object-name=s", # name of operand
					"object-type=s", # type of operand
					"action=s",		# action to perform
					"attribute=s",	# obj attributes
					"tier-type=s",	# type of clus / platform
					"task-id=s",	# task id
					"subtask-id=s",	# subtask id
					"source-cluster=s",	# source cluster
					"options=s",	# options
					"sync",			# write output to stdout/stderr
					"dbg"]);		# write debug logs

	my $clus_plat = "";

	if (defined $obj_data->{"tier-type"}) {
		$clus_plat = $obj_data->{"tier-type"};
	}

	if ((not defined $clus_plat) || length($clus_plat) <= 0) {
		fail_msg("Cannot determine the local clustering platform");
	}

	# import the cluster / appliance specific module
	{
		no strict 'refs';

		$tier_module = "VBSTier::$clus_plat";
		unless (eval "require $tier_module") {
			fail_msg("Failed to load module: $@");
		}

		$tier_module->import();
	}

	return $bin;
}

sub main() {
	# my $path = $Bin;
	my $path = init();
	my $object = {};
	my $fq_grpname = $obj_data->{"object-name"};
	my $dbg = "";

	### Remove this once VOM fixes it ###
	$ENV{"EAT_HOME_DIR"} = "";

	if (defined $fq_grpname) {
		($obj_data->{"local-cluster"}, $obj_data->{"object-name"}, undef) =
			split(/:/, $fq_grpname);
	}

	{
		no strict 'refs';
		$object = $tier_module->new($obj_data);
	}

	# enable debug logging if requested
	if (exists $object->{"dbg"}) {
		logdbg(1);
	}

	# run the operation requested
	my $ret_code = $object->run_operation();

	exit($ret_code);
}

main();

