#!/opt/VRTSsfmh/bin/perl
#
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $
#

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i) }

use strict;
use Getopt::Long;
use File::Spec;
use File::Copy;
use File::Path;
use JSON;
use Data::Dumper;
use CGI;

require VRTS::Ini;
require VRTS::Paths;
require VRTS::Util;
require VRTS::EXITCODE;
require VRTS::OS;

my $vbssvc_cmd;
my $logdir;
my $vbs_dir = undef;
my $vbsmain_json = 'vbsmain.json';
my $vbsmain_cf = 'vbsmain.cf';
my $vbs_etc_dir = undef;
my $logfile = 'notify_config_update.log';

sub init_logging
{
# Init the logger to create a log file loggin failures.

	my $log = new VRTS::Logger(
							Module => 'vbslog',
							Logdir => $logdir,
							Level => 'debug',
							Logfile => $logfile,
							UmiString => 'V-0000-0000');
}

sub discover_vbs_installation
{
	my $vbs_bin_dir = undef;

	if($^O =~ /Win32/i)
	{
		$vbs_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VRTSvbs\\Base" , "HomeDirectory");
		if (not defined $vbs_dir)
		{
			exit(&VRTS::EXITCODE::DIR_NOTFOUND);
		} 
		$vbs_bin_dir = $vbs_dir."\\bin\\";
		$logdir = $vbs_dir."\\log\\";
		$vbssvc_cmd  = $vbs_bin_dir."vbssvc.exe";
		$vbs_etc_dir = $vbs_dir."\\conf\\";
	} 
	else 
	{
		$vbssvc_cmd  = "/opt/VRTSvbs/bin/vbssvc";
		$logdir = "/var/VRTSvbs/log/";
		$vbs_etc_dir = "/etc/VRTSvbs/"
	}
	
	if ( ! -f "$vbssvc_cmd") 
	{
		exit(&VRTS::EXITCODE::FILE_NOTFOUND);
	}
	
	if($^O =~ /Win32/i)
	{
		$vbssvc_cmd = "\"" . $vbssvc_cmd . "\""; 
	}
}

sub copy_vbs_config
{
	my $src_file = undef;
	my $dest_file = undef;
	
	$src_file = File::Spec->catdir(VRTS::Paths::get_path("TempDir"), $vbsmain_cf);
	$dest_file = $vbs_etc_dir . $vbsmain_json;

	if ( ! -f "$src_file") 
	{
		VRTS::Logger::log('info', "Source configuration file \"$src_file\" not found. Exiting...");
		exit(&VRTS::EXITCODE::FILE_NOTFOUND);
	}
	
	copy($src_file, $dest_file) or return 1;
	
	VRTS::Logger::log('info', "Successfully copied configuration file $src_file to $dest_file");
	return 0;
}

sub main()
{
	init_logging();

	my $ret_copy = copy_vbs_config();
	exit (1) if $ret_copy;
	
	my $operation_type = "-notify-config-update";
	my $object_type = "ObjectCache";

	my $json = "{";
	$json .= "\"OperationType\" : \"$operation_type\", ";
	$json .= "\"ObjectType\" : \"$object_type\"";
	$json .= "}";
	
	$json = escape_json_string($json);

	my $cmd = "$vbssvc_cmd -send $json";

	my $retcode = execute_command($cmd);
	exit (1) if $retcode;
}

sub escape_json_string
{
	my $json = shift;
	
	if($^O =~ /Win32/i)
	{
		# Escaping quotes in the JSON message on Win32
		$json =~  s/"/\\"/g;
		# Adding quotes at the beginning and end
		$json = "\"$json\"";
	}
	else 
	{
		# Single quotes in case of Unix
		$json = "\'$json\'";
	}
	
	return $json;
}

sub execute_command
{
	my $cmd = shift;

	my $cout = `$cmd 2>&1`;
	my $ret = 0;
	chomp($cout);
	if($? != 0)
	{
		VRTS::Logger::log('info',"Command $cmd failed: $cout");
		print "Command failed : $cout\n";
		$ret = 1;
	}
	else 
	{
		VRTS::Logger::log('info',"Executed $cmd sucessfully with output: $cout");
		print "$cout\n";
		$ret = 0;
	}
}

discover_vbs_installation();
main();
exit(0);
