#!/opt/VRTSsfmh/bin/perl 
#$Id: vbs_show_sla.pl,v 1.3 2013/09/19 10:49:04 dgarai Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i) }

require VRTS::Ini;
require VRTS::Paths;
require VRTS::Util;
require VRTS::EXITCODE;
require VRTS::OS;
require VRTS::CMD;

my $hagrp_cmd;
my $hasys_cmd;
my $hares_cmd;
my $null_dev;

sub discover_vcs_installation
{
	my $vcs_dir = undef;
	my $vcs_bin_dir = undef;
	if($^O =~ /Win32/i)
	{
		$vcs_dir = VRTS::OS::querykey("Software\\VERITAS\\VCS\\Base" , "HomeDirectory");
		if (not defined $vcs_dir) {
				exit(&VRTS::EXITCODE::DIR_NOTFOUND);
		}
		$vcs_bin_dir = $vcs_dir."\\bin\\";
		$hagrp_cmd  = $vcs_bin_dir."hagrp.exe";
		$hasys_cmd  = $vcs_bin_dir."hasys.exe";
		$hares_cmd  = $vcs_bin_dir."hares.exe";
	} else {
		$hagrp_cmd = &VRTS::CMD::HAGRP;
		$hasys_cmd = &VRTS::CMD::HASYS;
		$hares_cmd = &VRTS::CMD::HARES;
		$logdir = "/var/VRTSvbs/log";
	}
	if ( ! -f "$hagrp_cmd") {
		exit(&VRTS::EXITCODE::FILE_NOTFOUND);
	}
	if ( ! -f "$hasys_cmd") {
		exit(&VRTS::EXITCODE::FILE_NOTFOUND);
	}
	if ( ! -f "$hares_cmd") {
		exit(&VRTS::EXITCODE::FILE_NOTFOUND);
	}
	if($^O =~ /Win32/i) {
		$hagrp_cmd = "\"" . $hagrp_cmd . "\"";
		$hasys_cmd = "\"" . $hasys_cmd . "\"";
		$hares_cmd = "\"" . $hasys_cmd . "\"";
	}
	if($^O =~ /Win32/i) {
		$null_dev = "nul";
	} else {
		$null_dev = "/dev/null"
	}
}

sub find_remote_cluster 
{
	my $sg = shift;
	my $ret = "-1";
	my $command = "$hagrp_cmd -display $sg -attribute ClusterList";
	my @result = `$command`;
	foreach my $val (@result) {
		if($val =~ /^\s*#/) {
		} else {
			my @values=split(' ', $val);
			if ($values[2] ne 'localclus') {
				return $values[2];
			}
		}
	}
	return ret;
}

sub find_compute_drsla_attr {
	my $res = shift;
	my $command = "$hares_cmd -value $res ComputeDRSLA";
	my @result_des=`$command >$null_dev 2>&1`;
	my $exit=$? >> 8;
	if ($exit eq 0) {
		return 0;
	}
	return -1;
}

sub find_remote_rpo_system {
	my $res = shift;
	my $remote_cluster = shift;
	my $ret = "-1";
	my $command = "$hasys_cmd -list -clus $remote_cluster";
	my @result = `$command`;
	for my $val (@result) {
		if($val =~ /^\s*#/) {
		} else {
			my @values=split(':', $val);
			chomp (@values[1]);	
			my $command = "$hares_cmd -action $res GetCurrentRPO -sys @values[1] -clus $remote_cluster";
			my @result_des=`$command >$null_dev 2>&1`;
			my $exit=$? >> 8;
			if ($exit eq 0) {
					return @values[1];
			}	
		}
	}
	return $ret; 
}

#Main
my @result_grps = split(' ', join(' ',@ARGV));
discover_vcs_installation();
for my $grps (@result_grps) {
	my $command="$hagrp_cmd -resources $grps";
	my @result_res=`$command`;	
	my $remote_cluster = find_remote_cluster($grps);		
	if ($remote_cluster eq "-1") {
		next;	
	}
	for my $res (@result_res) {
		if($res =~ /^\s*#/) {
		} else {
			chomp($res);
#Find the remote system names and where we 
#can calculate the RPO.	
			my $compute_drsla = find_compute_drsla_attr($res);
			if ($compute_drsla eq -1) {
				next;	
			}
			my $remote_sys = find_remote_rpo_system($res, $remote_cluster);	
			if ($remote_sys eq "-1") {
				next;	
			}
			my $command = "$hares_cmd -action $res GetCurrentRPO -sys $remote_sys -clus $remote_cluster";
			my @rpo_result;
			my $app_rpo="N/A";
			my @result_des=`$command`;
			my $rpo_flag = 0;
			shift(@result_des);
			for my $result_desired (@result_des) {
				chomp($result_desired);
				if ($result_desired =~ /^Application/) {
					if ($result_desired =~ /.*=\s+(\S+).*/) { 
						$app_rpo = $1;
					} 
				} else {
					if ($result_desired =~ /\S+/ and 
						$result_desired !~ /^Replication Lag Info/) {
						push(@rpo_result,$result_desired);
						push(@rpo_result,"\n");	
					}
				}	
			}
			print "$grps=$app_rpo=@rpo_result";
			print "\n";
		}	
	}
}
