#!/opt/VRTSsfmh/bin/perl

# $Copyrights: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

# MSG_ID Range : 10000 - 10099
# LAST_ID : 10008

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules")  if ($^O !~ /Win32/i) }
use strict;
use Getopt::Long;
use File::Spec;
use File::Copy;
use File::Path;

require VRTS::Paths;
require VRTS::Logger;
require VRTS::EXITCODE;
require VRTS::OS;

# Usage:
# vbsaction -recovery -parent <parentSG> -child <childSG> -dep <DependencyType> -user <user> -pwd <pwd> -domain <domain> -file script_file [script_file_argument(s)]

my ($vcs_dir, $vcs_root_dir, $vbs_dir, $vbs_var_dir, $vcs_home, $vbs_home, $vcs_bin_dir, $vbs_bin_dir, $logdir, $vbs_log);
my ($HACLUS, $HAGRP ,$HACLI, $VBSACTION, $CRA, $CFA);
my ($action_script_path);
my ($child_tier, $parent_tier, $dependency, $user, $domain, $pwd, $local, $parent_sg, $child_sg, $parent_clus, $child_clus, $action_type, $default_user);
my ($sfmh_perl, $lsystem_drive, $devnull);

sub init
{
	if ($^O =~ /Win32/i) {
		$devnull = "NUL";
		$vcs_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VCS\\Base" ,"HomeDirectory");
		$vcs_root_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VCS\\Base" ,"RootDirectory");
		$vbs_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VRTSvbs\\Base" ,"HomeDirectory");
		$vbs_var_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VRTSvbs\\Base" ,"VarDirectory");
		#$sfmh_perl = VRTS::OS::querykey("SOFTWARE\\Symantec\\SFMH" ,"InstallDir");
		$lsystem_drive = $ENV{"SystemDrive"} || "c:";
		#Use 8.3 path for perl.exe. Prepend drive letter later
		$sfmh_perl = "\\PROGRA~1\\VERITAS\\VRTSsfmh\\bin\\perl.exe";

		if (not defined $vcs_dir || not defined $vbs_dir || not defined $vbs_var_dir) {
			 exit(&VRTS::EXITCODE::DIR_NOTFOUND);
		}
		$vcs_home=$vcs_dir;
		$vbs_home=$vbs_dir;
		chop($vcs_dir) if (substr($vcs_dir, -1) eq "\\");
		chop($vbs_dir) if (substr($vbs_dir, -1) eq "\\");
		$vcs_bin_dir = $vcs_dir."\\bin\\";
		#Use 8.3 path for vbsaction.pl 
		$vbs_bin_dir=$lsystem_drive."\\PROGRA~1\\VERITAS\\VRTSvbs\\bin\\";
		$logdir = $vbs_dir."\\log\\";
		$HACLUS = "\"".$vcs_bin_dir."haclus.exe"."\"";
		$HAGRP = "\"".$vcs_bin_dir."hagrp.exe"."\"";
		$HACLI = "\"".$vcs_bin_dir."hacli.exe"."\"";
		$VBSACTION = $sfmh_perl." ".$vbs_bin_dir."vbsaction.pl"; #Prepend drive letter later
		$default_user = "system";
		$user = $default_user;
	} else {
		$devnull = "/dev/null";
		$vcs_home=$ENV{"VCS_HOME"} || "/opt/VRTSvcs";
		$vbs_home=$ENV{"VBS_HOME"} || "/opt/VRTSvbs";
		$vbs_log=$ENV{"VBS_LOG"} || "/var/VRTSvbs";
		$logdir = "$vbs_log/log";
		$HACLUS = "$vcs_home/bin/haclus";
		$HAGRP = "$vcs_home/bin/hagrp";
		$HACLI = "$vcs_home/bin/hacli";
		$VBSACTION = "$vbs_home/bin/vbsaction.pl";
		$default_user = "root";
		$user = $default_user unless $user;
	}
	init_logging();
	init_daemon_logging();
	return (0);
}


sub init_logging
{
	my $log = new VRTS::Logger(Module => 'vbsaction',
								Logdir => $logdir,
								Level => 'debug',
								Logfile => 'vbsaction.log',
								Logsize => 1024*1024,
								UmiString => 'V-0000-0000');
}


#Init for logging to vbsd engine log
sub init_daemon_logging
{
	if ($^O =~ /Win32/i) {
		my $dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VRTSvbs\\Base", "HomeDirectory");
		push (@INC, File::Spec->catpath(undef, $dir, "lib"));
	} else {
		push (@INC, "/opt/VRTSvbs/lib");
	}
	require VBSTier::Common;
	{
		no strict 'refs';
		my $mod = "VBSTier::Common";
		$mod->import();
	}
}


sub check_hacli
{
	my ($out, $ret);
	
	$out = `$HACLUS -value HacliUserLevel`;
	$ret = $? >> 8;
	chomp ($out);
	if ($out ne "COMMANDROOT") {
		VRTS::Logger::log('error', "Cluster attribute 'HacliUserLevel' must be set to 'COMMANDROOT' for script execution.");
		vbs_log_msg("E", "Cluster attribute 'HacliUserLevel' must be set to 'COMMANDROOT' for script execution.", 10000);
		$ret = 1;
	} 
	return $ret;
}

sub get_system_drives
{
	my %hash = ();
	if ($^O =~ /Win32/i) {
		my $out = `hacli -cmd "echo %SystemDrive%"`;
		%hash = split /\n+/, $out;
		#Remove the ':' that hacli appends to the system names
		for my $sys (keys(%hash)) {
			(my $new_sys = $sys) =~ s/:$//;
			$hash{$new_sys} = delete($hash{$sys});
		}	
	} 
	return \%hash;
}

sub run_script
{
	my ($out, $command_line);
	my (@online_nodes, @partial_nodes, @offline_nodes, @custom_script);
	my $ret = 1;
	if ($local) {
		#Run the script locally
		@custom_script = split(' ',$action_script_path);
		if (-x $custom_script[0] || $^O =~ /Win32/i) {
			$ENV{"VBS_SCRIPT_PARENT_TIER"} = "$parent_tier";
            $ENV{"VBS_SCRIPT_CHILD_TIER"} = "$child_tier";
            $ENV{"VBS_SCRIPT_DEPENDENCY_TYPE"} = "$dependency";
			if (defined $user && $^O !~ /Win32/i) {
				`su $user -c "$action_script_path >$devnull 2>&1"`;
			} else {
				`$action_script_path >$devnull 2>&1`;
			}
			$ret = $? >> 8;
			if ($ret) {
				VRTS::Logger::log('error', "Error executing '$action_script_path' for parent tier '$parent_tier', child tier '$child_tier', dependency '$dependency' as user '$user' on local system. Exit code: '$ret'.");
				vbs_log_msg("E", "Error executing '$action_script_path' for parent tier '$parent_tier', child tier '$child_tier', dependency '$dependency' as user '$user' on local system. Exit code: '$ret'.",
							10001, 
							"$action_script_path", "$parent_tier", "$child_tier", "$dependency", "$user", "$ret");
			} else {
				VRTS::Logger::log('info', "Executed '$action_script_path' for parent tier '$parent_tier', child tier '$child_tier', dependency '$dependency' as user '$user' on local system. Exit code: '$ret'.");
				vbs_log_msg("I", "Executed '$action_script_path' for parent tier '$parent_tier', child tier '$child_tier', dependency '$dependency' as user '$user' on local system. Exit code: '$ret'.",
							10002,
							"$action_script_path", "$parent_tier", "$child_tier", "$dependency", "$user", "$ret");
			}
			#TODO: Log the return code in vbs engine log
		} else {
			VRTS::Logger::log('error', "'$custom_script[0]' script does not exist or is not executable.");
			vbs_log_msg("E", "'$custom_script[0]' script does not exist or is not executable.", 10003, "$custom_script[0]");
		}
		
	} else {
		# Find the nodes where the script is to be executed
		my ($online_ref, $partial_ref, $offline_ref) = get_sg_state($parent_sg);
		@online_nodes = @$online_ref;
		@partial_nodes = @$partial_ref;
		@offline_nodes = @$offline_ref;
		if ($action_type eq $CRA) {
			my $flag;
			my $nodecount = scalar(@online_nodes) + scalar(@partial_nodes);
			if ($nodecount > 0) {
				my %system_drive_hash = %{get_system_drives()};
				# Execute the script on all the online and partial nodes
				foreach my $node (@online_nodes, @partial_nodes) {
					my $command_line = "$VBSACTION -local @ARGV"; 
					if ($^O =~ /Win32/i) {
						if ($system_drive_hash{$node}) {
								$command_line = $lsystem_drive.$command_line;
						} else {
								$command_line = $system_drive_hash{$node}.$command_line;
						}
					}
					VRTS::Logger::log('info', "Executing '$action_type' for parent tier '$parent_tier', child tier '$child_tier', dependency '$dependency' as user '$user' on system '$node'.");
					vbs_log_msg("I", "Executing '$action_type' for parent tier '$parent_tier', child tier '$child_tier', dependency '$dependency' as user '$user' on system '$node'.", 
					10004, "$action_type", "$parent_tier", "$child_tier", "$dependency", "$user", "$node");
					$out = `$HACLI -cmd "$command_line" -sys $node 2>&1`;
					$ret = $? >> 8;
					chomp ($out);
					if ($ret) {
						VRTS::Logger::log('error', "Error executing '$action_script_path' for parent tier '$parent_tier', child tier '$child_tier', dependency '$dependency' as user '$user' on system '$node'. Exit code: '$ret'. Output: '$out'");
						vbs_log_msg("E", "Error executing '$action_script_path' on system '$node'. Exit code: '$ret'.",
									10005,
									"$action_script_path", "$node", "$ret");
					} else {
						VRTS::Logger::log('info', "Executed '$action_script_path' for parent tier '$parent_tier', child tier '$child_tier', dependency '$dependency' as user '$user' on system '$node'. Exit code: '$ret'.");
						vbs_log_msg("I", "Executed '$action_script_path' on system '$node'. Exit code: '$ret'.",
									10006,
									"$action_script_path", "$node", "$ret");
					}
				}
			} else {
				VRTS::Logger::log('info', "Service Group '$parent_sg' is not online on any system. Hence '$action_type' is not executed.");
				vbs_log_msg("I", "Service Group '$parent_sg' is not online on any system.  Hence '$action_type' is not executed.", 
								10007, "$parent_sg", "$action_type");
			}
		} else {
			#For future release
			VRTS::Logger::log('info', "Unsupported action '$action_type'.");
			vbs_log_msg("I", "Unsupported action '$action_type'.", 10008, "$action_type");
		}
	}
	return $ret;
}

sub get_sg_state
{
	my $grp = $_[0];
	my (@online_list, @partial_list, @offline_list);

	my $out = `$HAGRP -state $grp -localclus`;
	my $ret = $? >> 8;
	chomp ($out);
	my @outlines = split('\n', $out);
	foreach my $val (@outlines) {
		if ($val =~ m/State\s+/) {
			my @elements = split ('\s+', $val);

			if ($elements[3] =~ m/ONLINE/) {
				push (@online_list, $elements[2]);
			} elsif ($elements[3] =~ m/OFFLINE/) {
				push (@offline_list, $elements[2]);
			} elsif ($elements[3] =~ m/PARTIAL/) {
				push (@partial_list, $elements[2]);
			}
		}
	}
	return (\@online_list, \@partial_list, \@offline_list);
}

sub process_args
{
	my $skip_count = 0;
	if ($ARGV[0] eq "-local") {
		$local = 1;
		shift @ARGV;
	}
	if ($ARGV[0] eq "-recovery") {
		$action_type = $CRA;
	} elsif  ($ARGV[0] eq "-fault") {
		$action_type = $CFA;
	}
	if (@ARGV < 9) {
		print_usage();
		return (1);
	}
	if ($ARGV[1] eq "-parent") {
		$parent_tier = $ARGV[2];
		($parent_clus, $parent_sg) = split(/:/,$parent_tier);
	}
	if ($ARGV[3] eq "-child") {
		$child_tier = $ARGV[4];
		($child_clus, $child_sg) = split(/:/,$child_tier);
	}
	if ($ARGV[5] eq "-dep") {
		$dependency = $ARGV[6];
	}

	if ($ARGV[7] eq "-user") {
		$user = $ARGV[8];
		$skip_count+=2;
	}

	#For future use	
	if ($ARGV[9] eq "-pwd") {
		$pwd = $ARGV[10];
		$skip_count+=2;
	}
	if ($ARGV[11] eq "-domain") {
		$domain = $ARGV[12];
		$skip_count+=2;
	}

	if ($ARGV[7+$skip_count] eq "-file") {
		$action_script_path = "@ARGV[8+$skip_count..$#ARGV]";
	}
	if (not defined $parent_tier || not defined $child_tier || not defined $dependency || not defined $action_type) {
		print_usage();
		return(1);
	}
}

sub print_usage
{
	print "Usage: vbs_action -recovery -parent <parentSG> -child <childSG> -dep <DependencyType> -user <user> -domain <domain> -pwd <pwd>\n";
}


sub main
{
	$CRA = "child_recovery_action";
	$CFA = "child_fault_action";
	exit (1) if process_args();
	exit (1) if init();
	exit (1) if check_hacli();
	exit (1) if run_script($local);
}

main(0);
exit(0);


