#!/opt/VRTSsfmh/bin/perl
#$Id: vbsdeploy.pl,v 1.52 2014/08/10 18:18:57 skembhav Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $
#

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules")  if ($^O !~ /Win32/i) }

use strict;
use Getopt::Long;
use File::Spec;
use File::Copy;
use File::Path;
use File::Temp qw(tempdir);
use JSON;
use Data::Dumper;
use CGI;
use Cwd;

require VRTS::Ini;
require VRTS::Paths;
require VRTS::Util;
require VRTS::EXITCODE;
require VRTS::OS;

my $vcs_home;
my $vbs_home;
my $vbs_log;
my $vbs_conf;
my $sfmh_dir;
my ($atutil_command, $vssat_command, $xprtlc_command);
my ($HASYS, $HAGRP, $HARES, $HACONF, $HACLUS);
my ($startprogram, $monitorprocess, $stopprogram);
my $eat_data_dir;
my $bmcfile;
my $smcfile;
my $messages_dir;
my $had_bmcmap;
my $bmcmap_cmd;
my $bmcgen_cmd;
my $clusteruuidfile;
my $cred_conf_file;
my ($vcs_dir, $vcs_root_dir, $vbs_dir, $vcs_bin_dir, $vbs_bin_dir, $vbs_var_dir,
    $vbs_conf_dir, $vbs_embe_conf_file, $logdir, $sfmh_bin_dir, $vbs_conf_file,
	$s3vbs_mhfile, $s3vbs_vbsfile);

my $deploy = "-deploy";
my $help = "-help";
my $verify = "-verify";
my $genbmc = "-genbmc";
my $winupgrade = "-winupgrade";
my $force = "-force";
my $CSG_EXPR = "\\bClusterService\\b";
my $vbsres_expr = "\\bvbsapp\\b";
my $ipres = "csgip";
my $nicres = "csgnic";
my $ipres_expr = "\\bcsgip\\b";


my $VBSGRP = "VBSService";
my $VBSGRP_EXPR = "\\bVBSService\\b";
my $CSG = "ClusterService";
my $VBSHS = "VBSHAService";
my $vbsres = "vbsapp";
my $vbsstart = "vbsd";
my $vbsstop = "vbsstop";
my $vbsmonitor = "vbsmonitor";
my $vbsd = "vbsd.bin";
my $configurator_ips= "CONFIGURATOR_IPS";
my $cshostnames = "";
my $had = "had";
my $all = 0;
my $vbsconf_content = "";
my $ORL = 3;
my $ORI = 120;
my $VBS_RSL = 1;
my ($vip, $mask, $rgconic, $prefix_ipv6);
my (@SYS, $localnode, $syscount);
my (@CMDS, $_DO, $_DOE);
my $TRACE = 0;
my $RO = 0;
my $force_deploy = 0;
my $csg_syslist;
# Supported languages
my $VBS_LANG = "en";
our %MSG;
our %COMM;
our %FLAG;

sub _do {
	my($do,$die,$line,$ret, $pid);
	($do,$die) = (@_);
	_p($do) if ($TRACE);
	#Read the file line by line, place each line into variable $line
	# INPUT is the file handle
	if ($^O =~ /Win32/i) {
		# Use IPC::Open3 instead of Open, to capture stderr output on Windows.
		use IPC::Open3;
		$pid = open3(\*STDIN, \*INPUT, \*INPUT, $do) || die("command $do failed");
	} else {
		open(INPUT,"$do 2>&1 |") || die("command $do failed");
	}
	while ($line = <INPUT>) { chomp($line); $ret = ($ret) ? "$ret\n$line" : $line; }
	close INPUT;
	if ($^O =~ /Win32/i) {
		waitpid $pid, 0;
	}
	$_DOE=$? >> 8;
	$_DO = $ret;
	_p("$_DOE $ret") if ($TRACE);
	if ($_DOE) {
		if ($die) {
			_die("Command $do failed. exit code: $_DOE. Output: $_DO");
		}
	} 
	VRTS::Logger::log('info',"Executed $do sucessfully. exit code: $_DOE. Output: $_DO");
	return $ret;
}

sub _p
{
	VRTS::Logger::log('info',"$_[0]");
	print "$_[0]\n";
	
}

sub _die {
	VRTS::Logger::log('error',"$_[0]");
	print "$_[0]\n";
	exit 1;
}

sub run_cmds {
	my(@_CMDS) = @_;

	foreach my $cmd (@_CMDS) {
		if ($cmd =~ /^#/) {
			$cmd =~ s/^#/ /;
			_p();
			_p($cmd);
			next;
		}
		_do($cmd,1); 
	}
	@_CMDS=(); 
}

sub init_logging 
{
# Init the logger to create a log file loggin failures.
	my $log = new VRTS::Logger(Module => 'vbslog',
			Logdir => $logdir,
			Level => 'debug',
			Logfile => 'vbsdeploy.log',
			Logsize => 1024*1024,
			UmiString => 'V-0000-0000');
}

sub local_sys_name {
# find local system name
	my $hasys_output = _do("$HASYS -nodeid");
	if($_DOE) {
		if ($hasys_output =~ /10600/) {
			_p("Cannot continue because VCS is not running, please start VCS.");
			exit 1;
		}
		else {
			_p("Command '$HASYS -nodeid' failed with error: $hasys_output");
			exit 1;
		}
	}
	$_DO =~ /:\s(\S+)/;
	$localnode = $1;
	return $localnode;
}

sub check_if_had_running
{
	my $arg = $_[0];
	my $hasys_output = _do("$HASYS -nodeid");
	if($_DOE) {
		if ($hasys_output =~ /10600/) {
			_p("Process '$vcs_home/bin/$had' not running. Exiting VBS Configuration");
			exit 1;
		}
	}
}

sub dump_and_run_cmds
{
	if ($RO) {	
		push(@CMDS, "$HACONF -dump -makero");
	} else {
		push(@CMDS, "$HACONF -dump");
	}

	run_cmds(@CMDS);
}

sub start_vbsd
{
	my ($tier_type) = shift;

	if ($tier_type eq "MSFOC" or $tier_type eq "MSCS") {
		start_vbsd_on_msfoc();
	} else {
		# VCS
		start_vbsd_on_vcs();
	}
}

sub start_vbsd_on_vcs
{
	my $sys;
	# check if ClusterService Group is online
	_do("$HARES -state $vbsres");
	if ($_DO =~ /ONLINE/) {
		_p("Virtual Business Services Application resource ($vbsres) is already online.")
	} else {
		_p("Onlining $CSG group");

		# wait for group to get probed on all systems
		foreach $sys (@SYS) {
			next if (defined $csg_syslist && $csg_syslist !~ /$sys/);
			_do("$HARES -probe $vbsres -sys $sys"); 
			_do("$HAGRP -wait $CSG ProbesPending 0 -sys $sys -time 15", 1); 
		}

		_do("$HAGRP -online $CSG -any", 1);
		_p("$_DO");
	}
}

sub start_vbsd_on_msfoc
{
	my $pwshl_cmd = get_msfoc_cmdlet();
	my $query_state = $pwshl_cmd . " ResourceGroup query $VBSHS state < NUL";

	_do($query_state);
	if ($_DOE == 0 and $_DO =~ /Online/i) {
		_p("Virtual Business Services Application resource"
			. " ($vbsres) is already online.")
	} else {
		my $online_vbshs = $pwshl_cmd .
							" ResourceGroup online $VBSHS wait < NUL";
		_p("Onlining $CSG group");
		_do($online_vbshs);
		_p("$_DO")
	}
}


sub csg_configured
{
	my $ret = 0;
	_do("$HAGRP -list");
	if (($_DOE == 0) && ($_DO =~ /$CSG_EXPR/)) {
		$ret = 1;
	}
	return $ret;
}

sub check_ip_resource
{
	my $res = undef;
	_do("$HARES -list Group=$CSG Type=IP");
	if ($_DOE == 0) {
		# TODO: consider case when there is more than
		# one IP resource in ClusterService group
		my @tmp_a = split ' ', $_DO;
		$res = $tmp_a[0];
	} else {
		if ($^O !~ /Win32/i) {
			VRTS::Logger::log('info',"IP resource not configured");
		} else {
			# check if IPv6 resource is configured in windows
			_do("$HARES -list Group=$CSG Type=IPv6");
			if ($_DOE == 0) {
				my @tmp_a = split ' ', $_DO;
				$res = $tmp_a[0];
			} else {
				VRTS::Logger::log('info',"IP resource not configured");
			}
		}
	}
	return $res;
}

sub create_vbsapp_resource
{
	my ($val, $msg, $sys);
	# check if vbs application resource exists
	my $restype;

	if ($^O !~ /Win32/i) {
		$restype = "Application";
	} else {
		$restype = "Process";
	}
	
	_do("$HARES -list Group=$CSG Type=$restype");
	if (($_DOE == 0) && ($_DO =~ /$vbsres_expr/)) {

		_p("Configuring Virtual Business Services $restype resource $vbsres");
		# check if vbsres attributes are correct
		_do("$HARES -value $vbsres StartProgram");
		if ($_DO ne "$startprogram") {
			push(@CMDS, "$HARES -modify $vbsres StartProgram $startprogram");
		}
		
		_do("$HARES -value $vbsres StopProgram");
		if ($_DO ne "$stopprogram") {
			push(@CMDS, "$HARES -modify $vbsres StopProgram $stopprogram");
		}

		if ($restype eq "Application") {
			_do("$HARES -value $vbsres MonitorProcesses");
			if ($_DO ne "$monitorprocess") {
				push(@CMDS, "$HARES -modify $vbsres MonitorProcesses $monitorprocess");
			}
		}

		# check for RestartLimit value
		_do("$HARES -display -ovalues $vbsres -group $CSG -type $restype", 1);
		if ($_DO =~ /RestartLimit/) {
			$val = _do("$HARES -value $vbsres RestartLimit");
			chomp($val);
			push(@CMDS, "$HARES -modify $vbsres RestartLimit $VBS_RSL") if ($val != $VBS_RSL);
		} else {
		# need to override RestartLimit
			push(@CMDS, "$HARES -override $vbsres RestartLimit");
			push(@CMDS, "$HARES -modify $vbsres RestartLimit $VBS_RSL");
		}

		# add dependency on ip resource
		$ipres = check_ip_resource();
		if (defined $ipres) {
			_do("$HARES -dep $vbsres");
			if ($_DO !~ /$vbsres\s+$ipres/) {
				push(@CMDS, "$HARES -link $vbsres $ipres");
			}
		}
		foreach $sys (@SYS) {
			next if (defined $csg_syslist && $csg_syslist !~ /$sys/);
			push(@CMDS, "$HARES -probe $vbsres -sys $sys"); 
		}
		return;
	} else {
	#	$msg = _tr("Adding Application resource $vbsres to group $CSG\n", 2000, 1144, "$vbsres", "$CSG");
	#	push(@CMDS, "#$msg");
		_p("Adding $restype resource $vbsres to group $CSG");
		push(@CMDS, "$HARES -add $vbsres $restype $CSG");	
		push(@CMDS, "$HARES -modify $vbsres Enabled 1");	
		push(@CMDS, "$HARES -modify $vbsres StartProgram $startprogram");
		push(@CMDS, "$HARES -modify $vbsres StopProgram $stopprogram");
		if ($restype eq "Application") {
			push(@CMDS, "$HARES -modify $vbsres MonitorProcesses $monitorprocess");
		}
		push(@CMDS, "$HARES -override $vbsres RestartLimit");
		push(@CMDS, "$HARES -modify $vbsres RestartLimit $VBS_RSL");
		$ipres = check_ip_resource();
		if (defined $ipres) {
			push(@CMDS, "$HARES -link $vbsres $ipres");
		}
		foreach $sys (@SYS) {
			next if (defined $csg_syslist && $csg_syslist !~ /$sys/);
			push(@CMDS, "$HARES -probe $vbsres -sys $sys"); 
		}
	}

}

sub create_csg_group
{
	my ($sys, $syslist, $astartlist, $orl, $ori, $msg, $csg_online, $auto_add_sys);

	# check if ClusterService Group exists 

	_do("$HAGRP -list");
	if (($_DOE == 0) && ($_DO =~ /$CSG_EXPR/)) {

		# check if ClusterService Group is online
		_do("$HAGRP -state $CSG");
		$csg_online = ($_DO =~ /ONLINE/);

		_p("Configuring existing $CSG ServiceGroup");
		# make sure the following are set correct
		
		# Do not modify SystemList and AutoStartList if AutoAddSystemToCSG is 0
		$auto_add_sys = _do("$HACLUS -value AutoAddSystemToCSG -localclus");
		if (($_DOE != 0) || ($auto_add_sys == 1)) {
			# * SystemList
			my $j=0;
			$syslist = _do("$HAGRP -value $CSG SystemList");
			foreach $sys (@SYS) {
				next if ($syslist =~ /$sys/);
				push(@CMDS, "$HAGRP -modify $CSG SystemList -add $sys $j");
				$j++;
			}
	
			# * AutoStartList
			$astartlist = _do("$HAGRP -value $CSG AutoStartList");
			foreach $sys (@SYS) {
				next if ($astartlist =~ /$sys/);
				push(@CMDS, "$HAGRP -modify $CSG AutoStartList -add $sys");
			}
		} else {
			VRTS::Logger::log('info',"AutoAddSystemToCSG attribute is set to $auto_add_sys.".
					" SystemList and AutoStartList of CSG will not be modified.");
			$csg_syslist = _do("$HAGRP -value $CSG SystemList");
		}
		# * OnlineRetryLimit 
		$orl = _do("$HAGRP -value $CSG OnlineRetryLimit");
		chomp($orl);
		push(@CMDS, "$HAGRP -modify $CSG OnlineRetryLimit $ORL") if ($orl != $ORL); 
		# * OnlineRetryInterval
		$ori = _do("$HAGRP -value $CSG OnlineRetryInterval");
		chomp($ori);
		push(@CMDS, "$HAGRP -modify $CSG OnlineRetryInterval $ORI") if ($ori != $ORI); 
	} else {

		my $i=0;
		my @sysarray;
		while ($i < @SYS) {
			$sysarray[$i] = $SYS[$i]." $i ";
			$i++;
		}
		$syslist = join(" ", @sysarray);
		$astartlist = join(" ", @SYS);

		_p("Creating $CSG ServiceGroup");
		push(@CMDS, "#$msg");
		push(@CMDS, "$HAGRP -add $CSG");
		
		# Do not modify SystemList and AutoStartList if AutoAddSystemToCSG is 0
		$auto_add_sys = _do("$HACLUS -value AutoAddSystemToCSG -localclus");
		if (($_DOE != 0) || ($auto_add_sys == 1)) {
			push(@CMDS, "$HAGRP -modify $CSG SystemList $syslist"); 
			push(@CMDS, "$HAGRP -modify $CSG AutoStartList $astartlist");
		} else {
			VRTS::Logger::log('info',"AutoAddSystemToCSG attribute is set to $auto_add_sys.".
					" SystemList and AutoStartList of CSG will not be modified.");
			$csg_syslist = _do("$HAGRP -value $CSG SystemList");
		}
		push(@CMDS, "$HAGRP -modify $CSG OnlineRetryLimit $ORL"); 
		push(@CMDS, "$HAGRP -modify $CSG OnlineRetryInterval $ORI"); 
		# indicate to online group
		$csg_online = 1;	
	}
	
}

sub add_vbs_res
{
	my ($tier_type) = shift;

	if ($tier_type eq "MSFOC" or $tier_type eq "MSCS") {
		my $pwshl_cmd = get_msfoc_cmdlet();
		$pwshl_cmd .= " ResourceGroup Create < NUL";
		# The default values of the following suffice:
		#    RestartThreshold
		#	 RestartPeriod
		#	 RestartAction
		# TODO: Need to understand RetryPeriodOnFailure better.
		_do($pwshl_cmd);
		if ($_DOE != 0) {
			VRTS::Logger::log('info',"$VBSHS could not be created.");
		}
	} else {
		# VCS
		create_csg_group();
		create_vbsapp_resource();
	}
}

sub make_command
{
	my @arr = @_;
	for (@arr)
	{
		$_ = qq["$_"] if /\s/;
	}
	my $_command_to_execute = join(' ',@arr);
	return ($_command_to_execute);
}

sub generate_bmcmap
{
	my ($tier_type) = shift;
	my $had_bmcmap_prev = $had_bmcmap.".prev";
	my $curr_dir = cwd();
	
	if ($^O =~ /Win32/i) {
		if ( ! -d "$messages_dir" )
		{
			VRTS::Logger::log('info',"Failed to update BMC map. " .
								"Messages directory '$messages_dir' does not exist");
			return 1;
		}
		if ( ! -f "$smcfile" ) 
		{
			VRTS::Logger::log('info',"Cannot generate BMC file. SMC file does not exist");
			return 1;
		}
		# Invoke the bmcgen command
		_do("$bmcgen_cmd -verbose -ascii \"$smcfile\"");
		
		if ( ! -f "$bmcfile" ) 
		{
			VRTS::Logger::log('info',"Failed to generate BMC file for VBS");
			return 1;
		}
		
		# Copy vbs bmc file to VCS messages directory
		VRTS::Logger::log('info',"Copying vbs bmc file to $messages_dir");
		copy($bmcfile, $messages_dir);

		# Backup the original file in case of failure
		if ( -f "$had_bmcmap" )
		{
			move($had_bmcmap, $had_bmcmap_prev);
		}
		
		# Invoke the bmcmap command
		chdir($messages_dir);
		_do("$bmcmap_cmd -create $VBS_LANG HAD -path \"$messages_dir\"");
		chdir($curr_dir);
		
		# Restore backup file on failure, delete otherwise
		if ( -f "$had_bmcmap" )
		{
			VRTS::Logger::log('info',"BMC map updated successfully");
			unlink($had_bmcmap_prev);
			return 0;
		}
		else 
		{
			VRTS::Logger::log('info',"BMC map update failed. Reverting to previous BMC map");
			move($had_bmcmap_prev, $had_bmcmap);
			return 1;
		}
	}
}

sub is_local_OS_64bit {
	if ($^O !~ /Win32/i) {
		# Give a meaningful implementation
		# for non-Windows when needed
		return 0;
	}

    my $prog_x86=$ENV{"PROGRAMFILES(x86)"};

    if (defined $prog_x86 and length($prog_x86) > 0) {
        return 1;
    } else {
        return 0;
    }
}

sub get_pwshl_bin {
    my $pwshl_path = VRTS::OS::querykey(
        "SOFTWARE\\Microsoft\\PowerShell\\1\\PowerShellEngine",
        "ApplicationBase");

    if (defined $pwshl_path) {
        if (is_local_OS_64bit()) {
            $pwshl_path =~ s/SysWOW64/sysnative/i;
            $pwshl_path =~ s/System32/sysnative/i;
        }
        $pwshl_path .= "\\powershell.exe";
    } else {
        $pwshl_path = "powershell.exe";
    }

    return $pwshl_path;
}

my $pwshl_escape = sub {
	my ($str) = @_;

	$str =~ s/ /` /g;
	return $str;
};

sub get_msfoc_cmdlet {
    my $pwshl_bin = get_pwshl_bin();
    return "$pwshl_bin " .
			File::Spec->catpath(undef, $pwshl_escape->($vbs_bin_dir),
								"msfocCmdlets.ps1");
}



sub offline_vbsapp_on_vcs {
	# Check if vbsapp resource is already created and it is online 
	_do("$HARES -state $vbsres");
	if (($_DOE == 0) && ($_DO =~ /ONLINE/)) {
		VRTS::Logger::log('info',"$vbsres resource exists and it is also online.");
		VRTS::Logger::log('info',"Offlining $vbsres resource before cleaning cred directory");
		#find the system where vbsapp is online
		my @lines = split('\n', $_DO);
		foreach my $l(@lines) {
			if ($l =~ /State\s+(\S+)\s+ONLINE/) {
				my $sys = $1;
				_do("$HARES -offline $vbsres -sys $sys");
				_do("$HARES -wait $vbsres State OFFLINE -sys $sys -time 15"); 
				last;
			}
		}
	}
}

sub offline_vbsapp_on_msfoc {
	my $pwshl_cmd = get_msfoc_cmdlet();
	my $chk_res_cmd = $pwshl_cmd . " Resource query $vbsres state < NUL";

	_do($chk_res_cmd);
	if (($_DOE == 0) && ($_DO =~ /Online/i)) {
		VRTS::Logger::log('info', "$vbsres resource exists and it is "
								 . "also online.");
		VRTS::Logger::log('info', "Offlining $vbsres resource before "
								 . "cleaning cred directory");

		my $offline_vbshs_cmd = $pwshl_cmd . " ResourceGroup offline $VBSHS wait";
		_do($offline_vbshs_cmd);
		if ($_DOE != 0) {
			VRTS::Logger::log('info',"$VBSHS could not be brought offline.");
		}
	}
}

sub import_creds
{
	my ($file) = shift;
	my ($eat_dir) = shift;

	#clean existing creds
	rmtree($eat_dir);
	_p("Importing credentials");
	my $cmd = make_command($atutil_command, "import", "-z", $eat_dir, "-f",
							$file, "-p", "vcsvcs");
	_do($cmd);
	_p("$_DO");
	
	return $_DOE;
}

sub credential_setup
{
	my ($file) = shift;
	my ($eat_dir) = shift;
	my ($tier_type) = shift;

	if ($tier_type eq "MSFOC" or $tier_type eq "MSCS") {
		offline_vbsapp_on_msfoc();
	} else {
		offline_vbsapp_on_vcs();
	}

	if (import_creds($file, $eat_dir) != 0) {
		_p("Unable to import credentials.Exiting Virtual Business Services Configuration");
		exit 1;
	}
	dumpclusteruuid($eat_dir);
	high_cipher();
}

sub high_cipher()
{
       open(FILE, "<${cred_conf_file}") || die "File $cred_conf_file not found";
       my @lines = <FILE>;
       close(FILE);

       my @newlines;
       foreach(@lines) {
               $_ =~ s/\"SSLCipherSuite\"=\"HIGH:MEDIUM:\!eNULL:\!aNULL:\!SSLv2\"/\"SSLCipherSuite\"=\"HIGH:\!eNULL:\!aNULL:\!SSLv2\"/g;
               push(@newlines,$_);
       }

       open(FILE, ">${cred_conf_file}") || die "File $cred_conf_file not found";
       print FILE @newlines;
       close(FILE);
}

sub dumpclusteruuid()
{
	my ($eat_dir) = shift;
	$ENV{"EAT_DATA_DIR"} = $eat_dir;
	$ENV{"EAT_HOME_DIR"} = $sfmh_dir;
	_p("dumping clusteruuid");
	my $clusteruuid;
	my $cmd = make_command($vssat_command, "showcred");
	_do($cmd, 1);
	my @lines = split('\n', $_DO);
	foreach my $l(@lines) {
	        if ($l =~ /CN=(\S+)\/OU=vbs_domain\@(\S+)\//) {
			print "$1\n";
			$clusteruuid = $1;
			$cshostnames= $2;
			
		}
	}
   	open(INPUT1, "> ${clusteruuidfile}")
			|| _die("Could not open ${clusteruuidfile} file!");
	print INPUT1 $clusteruuid;
	close (INPUT1);
		
}

sub init_pwshl {
	my $ret_code = 0;
	my $pwshl_bin = get_pwshl_bin();
	my $pwshl_get_execpolicy_cmd = "$pwshl_bin Get-ExecutionPolicy < NUL";

	_do($pwshl_get_execpolicy_cmd);

	if ($_DO ne "RemoteSigned") {
		my $pwshl_set_execpolicy_cmd =
			"$pwshl_bin Set-ExecutionPolicy RemoteSigned < NUL";
		_do($pwshl_set_execpolicy_cmd);
		$ret_code = $_DOE;
	}

	return $ret_code;
}

sub verify_vbs_roles_on_msfoc
{
	# The following only verifies that
	#  there is a Role/ResourceGroup called VBSHAService
	#  and that there is a Resource inside it called vbsapp
	my $pwshl_cmd = get_msfoc_cmdlet();
	$pwshl_cmd .= " Resource verify $vbsres $VBSHS < NUL";
	_do($pwshl_cmd);

	return $_DOE;
}

sub get_credential_details
{
	my $eat_dir = shift;
	my ($deployed, $domain, $uuid);

	$ENV{"EAT_DATA_DIR"} = $eat_dir;
	$ENV{"EAT_HOME_DIR"} = $sfmh_dir;

	my $cmd = make_command($vssat_command, "showcred");
   	_do($cmd);
	my @lines = split('\n', $_DO);
	foreach my $l(@lines) {
		if ($l =~ /CN=(\S+)\/OU=vbs_domain\@(\S+)\//) {
			$domain = $2;
			$deployed = 1;
		}

		if ($deployed eq "1") {
			if ($l =~ m/UUID:\s+{(\S+)}/) {
				$uuid = $1;
				last;
			}	
		}
	}
	
	return ($deployed, $domain, $uuid);
}

sub verify_deployment
{
	my $eat_dir = shift;
	my $tier_type = shift;
	my $cred_file = shift;
	my ($deployed, $domain, $uuid);
	
	if (defined $cred_file) {
		# Return failure if new credentials do not match existing ones
		return (1) if (compare_credentials($cred_file, $eat_data_dir) == 1);
	} else {
		($deployed, $domain, $uuid) = get_credential_details($eat_dir);
		return (1) if (not defined $deployed);
	}

	# Print the configurator(domain) name
	print "$domain " if (defined $domain);

	if ($tier_type eq "MSFOC" || $tier_type eq "MSCS") {
		if (verify_vbs_roles_on_msfoc() != 0) {
			VRTS::Logger::log('info',"vbsapp resource does not exist.");
			print "unconfigured";
			return 1;
		}
	} else {
		# Check if vbsapp resource is created 
		my $restype;
		if ($^O !~ /Win32/i) {
			$restype = "Application";
		} else {
			$restype = "Process";
		}
    
		_do("$HARES -list Group=$CSG Type=$restype");
		if (($_DOE != 0) || ($_DO !~ /$vbsres_expr/)) {
			VRTS::Logger::log('info',"vbsapp resource does not exist.");
			print "unconfigured";
			return 1;
		}
	}
	
	print "configured";
	print " $uuid" if (defined $uuid);
	return 0;
}

sub compare_credentials
{
	my ($file) = shift;
	my ($eat_dir) = shift;
	
	VRTS::Logger::log('info', "Checking for any existing VBS credentials.");
	
	my ($deployed, $domain, $uuid) = get_credential_details($eat_dir);
	if (not defined $deployed) {
		VRTS::Logger::log('info', "No existing VBS credentials found.");
		return (1);
	}
	
	VRTS::Logger::log('info', "Existing credentials found. Comparing them with new credentials.");
	my $temp_cred_dir = tempdir( CLEANUP => 1 );
	if (import_creds($file, $temp_cred_dir) != 0) {
		VRTS::Logger::log('info', "Unable to import new credentials in temporary dir.");
		return (1);
	}
	
	my ($deployed_new, $domain_new, $uuid_new) = get_credential_details($temp_cred_dir);

	if ((defined $deployed) && (defined $deployed_new) && ($domain eq $domain_new) 
			&& ($uuid eq $uuid_new)) {
		VRTS::Logger::log('info', "Existing deployed credentials match new credentials.");
		return (0);
	} else {
		return (1);
	}
}

sub usage_error_exit {
	_p("Invalid number of arguments. Exiting Virtual ".
		"Business Services Configuration.");
	print_usage();
	exit 1;
}

sub chk_tier_type
{
	my ($tier_type) = @_;

	if ($tier_type eq "MSCS") {
		return "MSFOC";
	} else {
		return $tier_type;
	}
}

sub verify_arguments
{
	my $tier_type = "";
	my $num_args = scalar(@_);

	if ($ARGV[0] eq "$deploy") {
		if ($num_args < 2 || $num_args > 4) {
			usage_error_exit();
		} elsif ($num_args == 3) {
			if ($ARGV[2] eq "$force") {
				$force_deploy = 1;
				$tier_type = "VCS";
			} else {
				$tier_type = chk_tier_type($ARGV[2]);
			}
		} elsif ($num_args == 4) {
			$tier_type = chk_tier_type($ARGV[2]);
			if ($ARGV[3] eq "$force") {
				$force_deploy = 1;
			} else {
				usage_error_exit();
			}
		} else {
			$tier_type = "VCS";
		}
	} elsif ($ARGV[0] eq "$verify"
			|| $ARGV[0] eq "$genbmc"
			|| $ARGV[0] eq "$winupgrade") {
		if ($num_args< 1 || $num_args> 2) {
			usage_error_exit();
		} elsif ($num_args == 2) {
			$tier_type = chk_tier_type($ARGV[1]);
		} else {
			$tier_type = "VCS";
		}
	}

	return $tier_type;
}

sub init
{
	my ($tier_type) = @_;

	if ($^O =~ /Win32/i) {
		$vcs_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VCS\\Base" ,"HomeDirectory");
		$vcs_root_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VCS\\Base" ,"RootDirectory");
		$vbs_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VRTSvbs\\Base" ,"HomeDirectory");
		$vbs_var_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VRTSvbs\\Base" ,"VarDirectory");
		$sfmh_dir = VRTS::OS::querykey("SOFTWARE\\Symantec\\SFMH\\xprtl\\config" ,"eat_home");
		if (not defined $vbs_dir || not defined $vbs_var_dir
			|| not defined $sfmh_dir
			|| ($tier_type eq "VCS" && not defined $vcs_dir))
		{			
			exit(&VRTS::EXITCODE::DIR_NOTFOUND);
		}
		$eat_data_dir = $vbs_var_dir."\\vbsauth\\data";
		$cred_conf_file = $eat_data_dir."\\systemprofile\\VRTSatlocal.conf"; 
		$clusteruuidfile = $eat_data_dir."\\clusteruuidfile";
		$vcs_home=$vcs_dir;
		$vbs_home=$vbs_dir;
		$vbsstart = "vbsd.exe";
		chop($vcs_dir) if (substr($vcs_dir, -1) eq "\\");
		chop($vbs_dir) if (substr($vbs_dir, -1) eq "\\");
		chop($sfmh_dir) if (substr($sfmh_dir, -1) eq "\\");
		$vcs_bin_dir = $vcs_dir."\\bin\\";
		$vbs_bin_dir = $vbs_dir."\\bin\\";
		$sfmh_bin_dir = $sfmh_dir."\\bin\\";
		$logdir = $vbs_dir."\\log\\";
		$vbs_conf_dir= $vbs_dir."\\conf\\";
		$vbs_embe_conf_file= $vbs_conf_dir."vbs_embedx.conf";
		$vbs_conf_file= $vbs_conf_dir."vbs.conf";
		$atutil_command = $sfmh_bin_dir."atutil.exe";
		$vssat_command = $sfmh_bin_dir."vssatbin.exe";
		$xprtlc_command = "\"$sfmh_bin_dir"."xprtlc.exe\"";
		$s3vbs_mhfile=  $sfmh_dir."\\agentlet\\"."\\VRTS\\"."\\VCS\\"."S3vbs.pm";
		$s3vbs_vbsfile=  $vbs_dir."\\agentlet\\"."S3vbs.pm";
		$HASYS = "\"".$vcs_bin_dir."hasys.exe"."\"";
		$HAGRP = "\"".$vcs_bin_dir."hagrp.exe"."\"";
		$HARES = "\"".$vcs_bin_dir."hares.exe"."\"";
		$HACONF = "\"".$vcs_bin_dir."haconf.exe"."\"";
		$HACLUS = "\"".$vcs_bin_dir."haclus.exe"."\"";
		$startprogram = "\"".$vbs_bin_dir."vbsd.exe"."\"";
		$monitorprocess =  "\"".$vbs_bin_dir."vbsd.exe"."\"";
		$stopprogram =  "\"".$vbs_bin_dir."vbsd.exe -stop"."\"";
		if ($tier_type eq "VCS") {
			$bmcmap_cmd = "\"".$vcs_bin_dir."bmcmap.exe"."\"";
			$bmcgen_cmd = "\"".$vcs_bin_dir."bmcgen.exe"."\"";
		} else {
			$bmcmap_cmd = "\"".$vbs_bin_dir."bmcmap.exe"."\"";
			$bmcgen_cmd = "\"".$vbs_bin_dir."bmcgen.exe"."\"";
		}
		$bmcfile = $vbs_bin_dir."VRTSvcsVBS.bmc";
		$smcfile = $vbs_bin_dir."VRTSvcsVBS.smc";
		$messages_dir = $vcs_root_dir."\\messages\\".$VBS_LANG;
		$had_bmcmap = $messages_dir."\\HAD.bmcmap";
	} else {
		$sfmh_dir = "/opt/VRTSsfmh";
		$vcs_home=$ENV{"VCS_HOME"} || "/opt/VRTSvcs";
		$vbs_home=$ENV{"VBS_HOME"} || "/opt/VRTSvbs";
		$vbs_log=$ENV{"VBS_LOG"} || "/var/VRTSvbs";
		$vbs_conf=$ENV{"VBS_CONF"} || "/etc/VRTSvbs";
		$logdir = "$vbs_log/log";
		$vbs_embe_conf_file = "${vbs_conf}/vbs_embedx.conf";
		$vbs_conf_file = "${vbs_conf}/vbs.conf";
		$atutil_command = "/opt/VRTSsfmh/bin/atutil";
		$vssat_command = "/opt/VRTSsfmh/bin/vssatbin";
		$xprtlc_command = "/opt/VRTSsfmh/bin/xprtlc";
		$s3vbs_mhfile = "$sfmh_dir/agentlet/VRTS/VCS/S3vbs.pm";
		$s3vbs_vbsfile = "$vbs_home/agentlet/S3vbs.pm"; 
		$HASYS = "$vcs_home/bin/hasys";
		$HAGRP = "$vcs_home/bin/hagrp";
		$HARES = "$vcs_home/bin/hares";
		$HACONF = "$vcs_home/bin/haconf";
		$HACLUS = "$vcs_home/bin/haclus";
		$startprogram = "$vbs_home/bin/$vbsstart";
		$monitorprocess = "$vbs_home/bin/$vbsd";
		$stopprogram = "\""."$vbs_home/bin/vbsd -stop"."\"";
		$eat_data_dir="/var/VRTSvbs/vbsauth/data";
		$cred_conf_file= "${eat_data_dir}/root/.VRTSat/profile/VRTSatlocal.conf";
		$clusteruuidfile = "/var/VRTSvbs/vbsauth/data/clusteruuidfile";
	}
	init_logging();

	if ($tier_type eq "MSFOC" or $tier_type eq "MSCS") {
		init_pwshl();
	} else {
		if ($ARGV[0] ne	"$verify" && $ARGV[0] ne "$genbmc"
			&& $ARGV[0] ne "$winupgrade") {
			check_if_had_running();                      
			local_sys_name();                            

			# find all nodes in cluster                  
			_do("$HASYS -list -localclus");
			@SYS = split(/\s+/, $_DO);
			$syscount = scalar(split(/\s+/, $_DO));

			$RO = _do("$HACLUS -value ReadOnly");

			# make configuration read-write              
			push(@CMDS, "$HACONF -makerw") if ($RO == 1);
		}
	}
}

sub add_entry_in_file
{
	my ($file, $entry) = @_;
	search_replace_in_file($file, $entry, $entry);
}

sub dump_vbs_conf_data
{
	_p("dumping contents into vbs.conf file\n$vbsconf_content"); 
	open(INPUT1, "> ${vbs_conf_file}")
		|| _die("Could not open ${vbs_conf_file} file!");
	print INPUT1 $vbsconf_content;
	close (INPUT1);
}

sub update_vbsconf_for_eat_data_dir
{
	my $entry = undef;
	$entry = "eat_data_dir = $eat_data_dir;\n";
	$vbsconf_content = $vbsconf_content . $entry;
}

sub update_vbs_embedx_conf
{
	my $entry = undef;
	if ($^O =~ /Win32/i) {
		$entry = "eat_home $vbs_dir";
		search_replace_in_file($vbs_embe_conf_file, "eat_home ", $entry);
		$entry = "document_root $vbs_dir\\web";
		search_replace_in_file($vbs_embe_conf_file, "document_root ", $entry);
		$entry = "var_dir $vbs_var_dir";
		search_replace_in_file($vbs_embe_conf_file, "var_dir ", $entry);
		$entry = "access_log $vbs_dir\\log\\access.log";
		search_replace_in_file($vbs_embe_conf_file, "access_log ", $entry);
		$entry = "error_log $vbs_dir\\log\\error.log";
		search_replace_in_file($vbs_embe_conf_file, "error_log ", $entry);
		$entry = "pid_file $vbs_dir\\xprtld.pid";
		search_replace_in_file($vbs_embe_conf_file, "pid_file ", $entry);
	}
}

sub search_replace_in_file
{
    no warnings ;
    my (${file}, ${search}, ${replace}) = @_;
    my $input = ();
    my @inputs = ();
    my $isfound = 0;
    my $counter = 0;
    if ( ! -e ("${file}")) {
		if (create_file($file, $configurator_ips)) {
			_die("Unable to create file");	
		}
    }
    open(INPUT1, "< ${file}") || _die("Could not open ${file} file!");
    @inputs = <INPUT1>;
    close (INPUT1);
    foreach $input (@inputs) {
        if($input =~ /^${search}/) {
            if ("${replace}" eq "") {
                delete $inputs[$counter];
            } else {
                $input =~ s/^${search}.*/${replace}/g;
            }
            $isfound = 1;
            last;
        }
        $counter++;
    }
    if (! $isfound && ${replace} ne "") {
        push(@inputs, "${replace}\n");
    }
    open(INPUT1, "> ${file}") || _die("Could not open ${file} file!");
    print INPUT1 @inputs;
    close (INPUT1);
    return 0;
}

sub create_file
{
    my (${conf_file}, ${stub_entry}) = @_;
    my ($old_umask) = ("");


    $old_umask = umask;
    umask "066";
    open CONFHANDLE, ">$conf_file" || _die("Unable to open file $!\n");

	if ($stub_entry) {
		print CONFHANDLE "$configurator_ips\n";
	}
    umask $old_umask;
    close CONFHANDLE;

    if (! -e "$conf_file") {
        return 1;
    } else {
        return 0;
    }
}

sub get_cshostnames
{
	my ($line, $result, @file, @hostarray);
	$result = "";
	@file = `$xprtlc_command -l https://localhost:5634:/world/getvitals`;
	my $i = 0;
	foreach $line(@file) {
		if ($line =~ m/\"sfm:\/\/(.*?):/) {
			$hostarray[$i] = $1;
			$i++
		}
	}
	$result = join(" ", @hostarray);
	return $result;
}

sub set_cshostnames
{
	$vbsconf_content .= "$configurator_ips=$cshostnames" . "\n";
}

sub update_vbsconf_file_for_cs_hostname
{
	if ($cshostnames eq "") {
		_die("Invalid input. Central Server hostnames not found");
	}
	set_cshostnames($cshostnames);
}

sub update_vbsconf_file_for_config_params
{
	$vbsconf_content .= "FAULT_RESET_TIMEOUT=900" . "\n";
	$vbsconf_content .= "PING_RETRY_INTERVALS=5,60,300" . "\n";
	$vbsconf_content .= "PING_RETRY_FREQUENCY=3" . "\n";
	$vbsconf_content .= "TASK_TIMEOUT=7200" . "\n";
}

sub update_vbsconf_for_tier_type
{
	my ($tier_type) = @_;

	$vbsconf_content .= "TIER_TYPE=$tier_type\n";
}

sub update_conf_files
{
	_p("Updating configuration files");
	update_vbsconf_file_for_cs_hostname();
	update_vbsconf_file_for_config_params();
	update_vbsconf_for_tier_type(@_);
	update_vbsconf_for_eat_data_dir();
	dump_vbs_conf_data();
	update_vbs_embedx_conf();
}

sub link_new
{
	my ($from, $to) = @_;
	my $ret = 0;

	if (! -e $to ) {
		if ($^O =~ /Win32/i) {
			$ret = copy($from, $to);
			if (!$ret) {
				VRTS::Logger::log('error',"Failed to copy $from");
			}
		} else {
			my $cmd = "ln -s $from $to";
			_do($cmd);
			if ($_DOE) {
				VRTS::Logger::log('error',"Failed to link $from");
			}
		}
	}
}

sub create_link
{
	my ($tier_type) = @_;
    my $ret = 0;

	link_new($s3vbs_vbsfile, $s3vbs_mhfile);
}

sub print_usage
{
	_p("Usage:");
	_p("vbsdeploy $deploy <credential_xml_file>\[ <tier_type>\]\[ $force\]");
	_p("vbsdeploy $verify\[ <tier_type>\]");
	_p("vbsdeploy $genbmc\[ <tier_type>\]");
	_p("vbsdeploy $winupgrade\[ <tier_type>\]");
	_p("vbsdeploy [$help]");
}

sub process_args
{
	my $tier_type;

	if ($ARGV[0] eq "$deploy") {
		$tier_type = verify_arguments(@ARGV);
		init($tier_type);
		VRTS::Logger::log('info',"Invoking vbsdeploy $deploy");
		VRTS::Logger::log('info',"Force deploy option is $force_deploy");
		generate_bmcmap($tier_type);
		if (($force_deploy == 0) && 
				(verify_deployment($eat_data_dir, $tier_type, $ARGV[1]) == 0)) {
			_die("Existing VBS configuration is valid. Aborting deploy operation.");
		}
		credential_setup($ARGV[1], $eat_data_dir, $tier_type);
		update_conf_files($tier_type);
		add_vbs_res($tier_type);
		create_link($tier_type);
		if ($tier_type eq "VCS") {
			push(@CMDS, "$HAGRP -enableresources $CSG");
			dump_and_run_cmds();
		}
		start_vbsd($tier_type);
	} elsif ($ARGV[0] eq "$verify") {
		$tier_type = verify_arguments(@ARGV);
		init($tier_type);
		VRTS::Logger::log('info',"Invoking vbsdeploy $verify");
		create_link($tier_type);
		exit verify_deployment($eat_data_dir, $tier_type);
	} elsif ($ARGV[0] eq "$genbmc" || $ARGV[0] eq "$winupgrade") {
		$tier_type = verify_arguments(@ARGV);
		init($tier_type);
		VRTS::Logger::log('info',"Invoking vbsdeploy $ARGV[0]");
		update_vbs_embedx_conf();
		exit generate_bmcmap($tier_type);
	} elsif ($ARGV[0] eq "$help") {
		print_usage();
		exit 0;
	} else {
		print_usage();
		exit 1;
	}
}

# main()

process_args();

exit 0;

