#!/opt/VRTSsfmh/bin/perl
#$Id: vbsdr.pl,v 1.7 2012/09/24 09:15:58 dgarai Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

#1. It checks if this service group is online on the local cluster.
# If yes it will return.

#2. Second this script checks if the Global Group is actually a global group.
# For checking the 1 condition it used the command 
# hagrp -display SG1 -attribute Authority -clus localclus
# #Group       Attribute             System                Value
# SG1          Authority             global                1

#3. Third it checks if we need to check the authority. If we need to check
# authority then we use the following command.
# hagrp -display SG1 -attribute Authority -clus localclus
# #Group       Attribute             System                Value
# SG1          Authority             global                1
# The value of 1 means it has authority so we procced. In case we don't need
# to check the authority it means that someone has used -force command. 

#4. Fourth we need to check if the Global Service Group is online in any of the
# remote clusters. For that we use the following command.
# hagrp -state SG1
# Group       Attribute             System                Value
# SG1          State                 debavbs:rhel5vm2      |OFFLINE|
# SG1          State                 localclus:loose-node1 |ONLINE|
# If there are more than 2 DR site also it will work as this list will
# check if its online in any of the line.

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i) }

require VRTS::Ini;
require VRTS::Paths;
require VRTS::Util;
require VRTS::EXITCODE;
require VRTS::OS;
require VRTS::CMD;

my $hagrp_cmd;
my $haclus_cmd;

sub discover_vcs_installation
{
	my $vcs_dir = undef;
	my $vcs_bin_dir = undef;

	if($^O =~ /Win32/i)
	{
		$vcs_dir = VRTS::OS::querykey("Software\\VERITAS\\VCS\\Base" , "HomeDirectory");
		if (not defined $vcs_dir)
		{
			exit(&VRTS::EXITCODE::DIR_NOTFOUND);
		} 
		$vcs_bin_dir = $vcs_dir."\\bin\\";
		$hagrp_cmd  = $vcs_bin_dir."hagrp.exe";
		$haclus_cmd  = $vcs_bin_dir."haclus.exe";
	} 
	else 
	{
		$hagrp_cmd = &VRTS::CMD::HAGRP;
		$haclus_cmd = &VRTS::CMD::HACLUS;
		$logdir = "/var/VRTSvbs/log";
	}
	
	if ( ! -f "$hagrp_cmd") 
	{
		exit(&VRTS::EXITCODE::FILE_NOTFOUND);
	}

	if ( ! -f "$haclus_cmd") 
	{
		exit(&VRTS::EXITCODE::FILE_NOTFOUND);
	}

	if($^O =~ /Win32/i)
	{
		$hagrp_cmd = "\"" . $hagrp_cmd . "\""; 
		$haclus_cmd = "\"" . $haclus_cmd . "\""; 
	}
}

my $ret = 0;
sub check_value {
	my $value = shift;
	my $check = shift;
		
	printf "Values of function $value and $check\n";
	if ($value eq $check) {
		return -1;
	}
	return 0;
}

sub parse_value {
	my $command = shift;
	my $check = shift;
	my $index = shift;

	my @result = `$command`;
	foreach my $val (@result) {
		if($val =~ /^\s*#/) {
		} else {
			my @values=split(' ', $val);
			$ret = check_value($values[$index], $check);
			if ($ret == -1) {
				return $ret;
			}
		}
	}
	return 0;
}

sub check_globalclus_state {
	my $sg = shift;
	my $command = "$hagrp_cmd -display $sg -attribute ClusterList";
	my @result = `$command`;
	foreach my $val (@result) {
		if($val =~ /^\s*#/) {
		} else {
			my @values=split(' ', $val);
			if ($values[2] ne 'localclus') {
				printf "Cluster Name=$values[2]\n";
				my $command = "$haclus_cmd -state $values[2]";
				my $output = `$command`;
				chomp($output);
				if ($output eq "RUNNING") {
					$ret = -1;
					return $ret;
				}
			}	
		}
	}
	return $ret;
}

sub check_authority {
	my $sg = shift;
	my $command = "$hagrp_cmd -display $sg -attribute Authority";
	my @result = `$command`;
	foreach my $val (@result) {
		if($val =~ /^\s*#/) {
		} else {
			my @values=split(' ', $val);
			if ($values[3] ne 0) {
				printf "Cluster Name=$values[2] has Authority\n";
				$ret = -1;
				return $ret;
			}	
		}
	}
	return $ret;
}

sub check_globalsg_state {
	my $sg = shift;
	my $r_cluster = shift;
	my $ret = 0;
	my $command = "$hagrp_cmd -state $sg -clus $r_cluster";
	my @result = `$command`;
	foreach my $val (@result) {
		if($val =~ /^\s*#/) {
		} else {
			my @values=split(' ', $val);
			if ($values[3] eq '|ONLINE|') {
				$ret = -1;	
				return $ret;		
			}
		}
	}
	return $ret;
}

sub check_globalsg {
	my $sg = shift;
	my $command = "$hagrp_cmd -display $sg -attribute ClusterList";
	my @result = `$command`;
	foreach my $val (@result) {
		if($val =~ /^\s*#/) {
		} else {
			my @values=split(' ', $val);
			if ($values[2] ne 'localclus') {
				printf "Cluster Name=$values[2]\n";
				$ret = check_globalsg_state($sg, $values[2]);
				return $ret;		
			}	
		}
	}
	return $ret;
}


sub check_globalclus{
	my $check = shift;
	my $command = "$hagrp_cmd -value $check ClusterList";
	my @result = `$command`;
	my $count = 0;

	foreach my $val (@result) {
		if($val =~ /^\s*#/) {
		} else {
			my @values=split(' ', $val);
			foreach my $str (@values) {
				$count = $count + 1;
			}
		}
	}
	printf("value of count=$count\n");
	if ($count > 2) {
		#This is is global group
		$ret = -1;
	}
	return $ret;
}

my $globalsg = $ARGV[0];
my $option = $ARGV[1];
my $online = "|ONLINE|";
my $global = "global";
my $takeover = "-takeover";
my $globalstate = "globalstate";
my $authority = "-authority";
my $switch = "-switch";

if ($globalsg eq '') {
	printf "\nThere is no service group provided..exit\n";
	exit -1;
}
printf "Global Service Group:$globalsg\n";

discover_vcs_installation();

#if the option flag is enabled we only need to find out
#if the cluster on the DR side is Running or not.
#In case the global cluster is running then we need to
#-1 else return 0.
if ($option eq $takeover) {
	$ret = check_globalclus_state($globalsg);	
	if ($ret == 0) {
		printf("The Global Cluster is faulted: Return Success\n");	
		exit(0);
	} else {
		printf("The Global Cluster is Running\n");	
		exit(1);
	} 
}

if ($option eq $globalstate) {
	$ret = check_globalclus($globalsg);	
	if ($ret == 0) {
		printf("The Group is Local\n");	
		exit(0);
	} else {
		printf "The Group is Global\n";
		exit(1);
	}
}
 
if ($option eq $authority) {
	$ret = check_authority($globalsg);	
	if ($ret == 0) {
		printf("No Cluster has the authority set for Service Group $globalsg \n");	
		exit(0);
	} else {
		printf("Authority set for Service Group $globalsg \n");	
		exit(1);
	} 
}

if ($option eq $switch) {
	$ret =  check_globalsg($globalsg);
	if ($ret == -1) {
		printf("The Global Servcie Group is Running in the Remoter Cluster\n");
		exit(0);
	} else {
		printf("The Global Servcie Group is Not Running in the Remote Cluster\n");
		exit(1);
	}	
}


#Case 1. Check if Service Group is Global.
printf "\n\nChecking if the Service Group is Global";
$ret = check_globalclus($globalsg);	
if ($ret == 0) {
	printf "The Group is Local\n";
} else {
	printf "The Group is Global\n";
}

#Case 2. Check if service group online in the local clus.
printf "\n\nChecking the Global Service Group status in local clusters\n";
my $command = "$hagrp_cmd -state $globalsg -clus localclus";
printf "Command = $command\n";

$ret = parse_value($command, $online, 3); 

if ($ret == -1) {
	printf "The Service group is online\n";
	exit 0;
} else {
	printf "The Service group is offline\n";
}

#Case 3. Check if the Global Service Group has Authority in local Cluster
printf "\n\nChecking the Global Service Group has Authority\n";
my $command = "$hagrp_cmd -display $globalsg -attribute Authority -clus localclus";
printf "Command = $command\n";
$ret = parse_value($command, 1, 3);

if ($ret == 0) {
	printf "The local cluster don't have  authority\n";
	#Now check if the GCO is offline or not. In case of offline 
	#we can online the service group.
	$ret = check_globalclus_state($globalsg);	
	if ($ret == 0) {
		printf "The Global State is faulted. You need to run -force to grab the authority.\n";
		exit 0;
	} 
} else {
	printf "The local cluster has authority\n";
}


#Case 4. Check if the Global Service Group is online in any clusters.
printf "\n\nChecking the Global Service Group status in clusters\n";
my $command = "$hagrp_cmd -state $globalsg";
printf "Command = $command\n";
$ret = parse_value($command, $online, 3);

if ($ret == -1) {
	printf "The Global Service Group is online\n";
	exit -1;
} else {
	printf "The Global Service Group is offline\n";
} 

printf "\n\n\nHurry!!! You can start VBS \n";
exit 0;

