#!/opt/VRTSsfmh/bin/perl

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i) }
use strict;
use Data::Dumper;

require VRTS::Spoke;
require VRTS::Paths;
require VRTS::Logger;

my $vbs_conf_file = "";
my $log_dir = "";
my $vbssvc;
my $logobj;
my $operation = "";
my $vm_id = "";
my $sg_id = "";
my $vm_name = "";
my $task_id = "";
my $subtask_id = "";
my $source_clus = "";

if($^O =~ /Win32/i) {
        my $vbs_dir = VRTS::OS::querykey("SOFTWARE\\VERITAS\\VRTSvbs\\Base" , "HomeDirectory");
        if (not defined $vbs_dir)
        {
            VRTS::Logger::log('error', "VBS installation does not exist. Exiting...");
            exit;
        }
		my $vbs_conf_dir= $vbs_dir."\\conf\\";
		$log_dir = $vbs_dir."\\log\\";
		$vbs_conf_file= $vbs_conf_dir."\\vbs.conf";
		my $vbs_bin_dir = $vbs_dir."\\bin\\";
		$vbssvc  = $vbs_bin_dir."vbssvc.exe";
		$vbssvc = "\"" . $vbssvc . "\"";

} else {

	$vbs_conf_file = "/etc/VRTSvbs/vbs.conf";
	$log_dir = "/var/VRTSvbs/log";
	$vbssvc = "/opt/VRTSvbs/bin/vbssvc";
}

sub escape_json_string
{
	my $json = shift;
	
	if($^O =~ /Win32/i)
	{
		# Escaping quotes in the JSON message on Win32
		$json =~  s/"/\\"/g;
		# Adding quotes at the beginning and end
		$json = "\"$json\"";
	}
	else 
	{
		# Single quotes in case of Unix
		$json = "\'$json\'";
	}
	
	return $json;
}

sub vbs_log_initlize
{

$logobj = new VRTS::Logger(Module => 'vmware_client', Logdir => $log_dir,
                           Level => 'debug', Logfile => 'vmware_client.log',
                           UmiString => '');
}

vbs_log_initlize();
process_args();


my $cs_name = read_value_from_file($vbs_conf_file, "CONFIGURATOR_IPS");
if (!$cs_name) {
	VRTS::Logger::log('error', "Failed to get cs name");
	exit(0)
}

if ($vm_name ne "") {
	my $out = `$vbssvc -vminfo -details 2>&1`;
	my $ret = $?;
	chomp ($out);
	if ($ret) {
		VRTS::Logger::log('error', "Failed to execute command $out");
		exit 1;
	}
	my @outlines = split('\n', $out);
	foreach my $val (@outlines) {
		if ($val !~ m/Name\s+/) {
			my @elements = split ('\s+', $val);
			if (@elements  lt 5) {
				VRTS::Logger::log('error', "virtual machine $vm_name info is not correct");
				exit (1);
			}
			my $vm_value = $elements[3];
			if ($vm_name eq $vm_value) {
				$sg_id = $elements[2];
				$vm_id = $elements[4];
				last;
			}
		
		}
	}
	if ($sg_id ne "" && $vm_id ne "") {
		$operation = "VM_STATUS";
		VRTS::Logger::log('debug', "Found vm id $vm_id and sg id $sg_id for specified virtual machine $vm_name");
	} else {
		VRTS::Logger::log('debug', "virtual machine $vm_name details not found");
		exit(1);
	}
}

if ($operation =~ /START/) {
	#before starting check if VM is already started
	VRTS::Logger::log('debug', "Invoking operation:VM_STATUS for vm $vm_id and for sg $sg_id");
	my %vm_details = ("VM_ID" => $vm_id, "SG_ID" => $sg_id, "OP_NAME" => "VM_STATUS");
	my $cookie = VRTS::Spoke::get_cookie("VBS");
	my ($ret, $out) = VRTS::Spoke::execute($cs_name, $cookie, "VBS", \%vm_details, undef);
	$out =~ s/\n\s*/ /g;
	VRTS::Logger::log('debug', "return code: $ret out: $out");
	if ($out =~ /stderr.*ONLINE/) {
		VRTS::Logger::log('debug', "VM is already started");
		send_taskupdate();
		exit(0);
	}
}

VRTS::Logger::log('debug', "Invoking operation:$operation for vm $vm_id and for sg $sg_id");
my %vm_details = ("VM_ID" => $vm_id, "SG_ID" => $sg_id, "OP_NAME" => "$operation");
my $cookie = VRTS::Spoke::get_cookie("VBS");
my ($ret, $out) = VRTS::Spoke::execute($cs_name, $cookie, "VBS", \%vm_details, undef);
$out =~ s/\n\s*/ /g;
VRTS::Logger::log('debug', "return code: $ret out: $out");
send_taskupdate();

if (!$ret) {
	if ($operation =~ /STATUS/) {
	my $head = '{ "OperationType" : "-update", "ObjectType" : "VirtualMachine", "CLIOperation" : true, "Objects" : [';
	my $tail = '] }';

    my $json = '{' ;
    $json .= '"ObjectName" : "' . $vm_id . '",';
    $json .= ' "Attributes" : [ ';
    $json .= '{';
    $json .= ' "AttributeName" : "VMState", ';
	if ($out =~ /stderr.*ONLINE/) {
		$json .= '"Value" : ' . '4' ;
	} elsif ( $out =~ /stderr.*OFFLINE/) {
		$json .= '"Value" : ' . '2' ;
	} else {
		$json .= '"Value" : ' . '1' ;
	}
    $json .=  '}';
    $json .=  ']';
    $json .= '}';
	$json = $head . $json . $tail;
	VRTS::Logger::log('debug', "Status of $vm_id: $json");
	my $json_str = escape_json_string($json);
	my $cmdout = `$vbssvc -send $json_str 2>&1`;
	$cmdout =~ s/\n\s*/ /g;
	VRTS::Logger::log('debug', "cmd out: $cmdout");
	}
} 

sub read_value_from_file
{
    my (${file}, ${attribute})  = @_;
    my (@tags) = ();
    my ($name, $value, $input) = ("", "", "");
    my @inputs = ();
    open(INPUT1, "${file}") || die("Could not open $file!");
    @inputs = <INPUT1>;
    close (INPUT1);
    foreach $input (@inputs) {
        chomp ($input);

        # remove any beginning or trailing space and squeeze it.
        $input =~ s/\s+/ /g;
        $input =~ s/^\s+//;
        $input =~ s/\s+$//;
        if($input =~ /^${attribute}.*/) {
            @tags = split (/[=#;]/, $input);
            $name = $tags[0]; $name =~ s/^\s//;
            $value = $tags[1]; $value =~ s/^\s//;
            return ($value);
        }
    }
    return ("");
}

sub process_args
{
	if (@ARGV == 6) {
		$operation = $ARGV[0];
		$vm_id = $ARGV[1];
		$sg_id = $ARGV[2];
		$task_id = $ARGV[3];
		$subtask_id = $ARGV[4];
		$source_clus = $ARGV[5];
	} elsif (@ARGV == 1) {
		$vm_name = $ARGV[0];
	} elsif (@ARGV == 3) {
		$operation = $ARGV[0];
		$vm_id = $ARGV[1];
		$sg_id = $ARGV[2];
	} else {
		VRTS::Logger::log('debug', "Invalid arguments");
		exit(1);
	}
}
sub send_taskupdate
{
	if ($operation !~ /STATUS/) {
		my $operation_type = "-update-subtasks";
		my $object_type = "Task";
		
		my $json = "{";
		$json .= "\"OperationType\" : \"$operation_type\", ";
		$json .= "\"ObjectType\" : \"$object_type\", ";
		$json .= "\"CLIOperation\" : true, ";
		$json .= "\"Objects\" : [ ";
		$json .= " { ";
		$json .= "\"ObjectName\" : \"$task_id\", ";
		$json .= "\"Attributes\" : [ ";
		$json .= " { \"AttributeName\" : \"SubtaskId\", \"Value\" : $subtask_id }, ";
		$json .= " { \"AttributeName\" : \"SourceCluster\", \"Value\" : \"$source_clus\" }, ";
		$json .= " { \"AttributeName\" : \"ExitCode\", \"Value\" : $ret } ";
		$json .= " ] ";
		$json .= " } ";
		$json .= " ] ";
		$json .= "}";
		
		my $json_str = escape_json_string($json);
		my $cmdout = `$vbssvc -send $json_str 2>&1`;
		$cmdout =~ s/\n\s*/ /g;
		VRTS::Logger::log('debug', "cmd out: $cmdout");
	}
}

